-- by "Marhu" 
-- v 1.0
-- Date: 02.04.2013
-- KuhZuchtPlace (Zuchtstation Objekt)

source(g_currentModDirectory.."KuhZuchtUpdateFillLvlEvent.lua");
	
KuhZuchtPlace = {};
KuhZuchtPlace_mt = Class(KuhZuchtPlace, Placeable);
InitObjectClass(KuhZuchtPlace, "KuhZuchtPlace");

function KuhZuchtPlace:new(isServer, isClient, customMt)
    local self = Placeable:new(isServer, isClient, KuhZuchtPlace_mt);
    registerObjectClassName(self, "KuhZuchtPlace");
	
	return self;
end;

function KuhZuchtPlace:delete()
    if self.FillTriggers then
		for i = 1 , table.getn(self.FillTriggers) do
			removeTrigger(self.FillTriggers[i]);
		end;
	end;
	if KuhZuchtZE then
		for i = 1 , table.getn(KuhZuchtZE.KuhZuchtPlace) do
			if KuhZuchtZE.KuhZuchtPlace[i] == self then
				table.remove(KuhZuchtZE.KuhZuchtPlace,i);
				break;
			end;
		end;
	end;
	
	unregisterObjectClassName(self);
    KuhZuchtPlace:superClass().delete(self);
end;

function KuhZuchtPlace:deleteFinal()
    KuhZuchtPlace:superClass().deleteFinal(self);
end;

function KuhZuchtPlace:load(xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom)
    if not KuhZuchtPlace:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom) then
        return false;
    end;
	
	if not moveMode then
		self.xml = xmlFilename;
		self.pos = {x,y,z};
		self.rot = {rx,ry,rz};
		local xmlFile = loadXMLFile("TempXML", xmlFilename);
				
		local FillTrigger = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.FillTrigger#index"));
		if FillTrigger ~= nil then
			self.FillTriggers = {};
			for i = 1,getNumOfChildren(FillTrigger) do
				self.FillTriggers[i] = getChildAt(FillTrigger,i-1);
				addTrigger(self.FillTriggers[i], "TriggerCallback", self);
			end;
		end;
	
		local KaelberGroup = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.Kaelber#index"));
		self.Kaelber = {};
		for i = 1, 5 do
			self.Kaelber[i] = {};
			self.Kaelber[i].vis = getChildAt(KaelberGroup, i-1);
			self.Kaelber[i].has = false;
			self.Kaelber[i].t = -1;
		end;
		
		delete(xmlFile);
		
		self.updateMs = 0;
		self.updateMin = 0;
		self.numKalb = 0;
		self.SendAddCalb = false;
		
		self.setFillLevel = KuhZuchtPlace.setFillLevel;
        self.getFillLevel = KuhZuchtPlace.getFillLevel;
		self.addKalb = KuhZuchtPlace.addKalb;
		
		if KuhZuchtZE then		
			table.insert(KuhZuchtZE.KuhZuchtPlace, self);
		end;
		
		self.AddDirtyFlag = self:getNextDirtyFlag();
		
	end;
  
	return true;
end;

function KuhZuchtPlace:readStream(streamId, connection)
	local fillLevel = streamReadFloat32(streamId);
	local updateMin = streamReadFloat32(streamId);
	if KuhZuchtZE then
		 KuhZuchtZE.updateMin = updateMin;
	end
	local Kaelber = {};
	for i = 1, 5 do
		Kaelber[i] = {};
		Kaelber[i].has = streamReadBool(streamId);
		Kaelber[i].t = streamReadFloat32(streamId);
	end;
	KuhZuchtPlace:superClass().readStream(self, streamId, connection);
	for i = 1, 5 do
		self.Kaelber[i].has = Kaelber[i].has;
		self.Kaelber[i].t = Kaelber[i].t;
		if Kaelber[i].has then
			setVisibility(self.Kaelber[i].vis,true);
		end;
	end;
	self:setFillLevel(fillLevel, Fillable.FILLTYPE_MILK, true);
	
end;

function KuhZuchtPlace:writeStream(streamId, connection)
    	streamWriteFloat32(streamId, self:getFillLevel(Fillable.FILLTYPE_MILK));
		local updateMin = 0;
		if KuhZuchtZE and KuhZuchtZE.updateMin then
			updateMin = KuhZuchtZE.updateMin;
		end
		streamWriteFloat32(streamId, updateMin);
		for i = 1, 5 do
			streamWriteBool(streamId, self.Kaelber[i].has);
			streamWriteFloat32(streamId, self.Kaelber[i].t);
		end
    KuhZuchtPlace:superClass().writeStream(self, streamId, connection);
end;

function KuhZuchtPlace:readUpdateStream(streamId, timestamp, connection)
	if connection:getIsServer() then
		local AddKalb = streamReadBool(streamId);
		if AddKalb then
			self:addKalb();
		end;
	end;
	KuhZuchtPlace:superClass().readUpdateStream(self, streamId, timestamp, connection);
end;

function KuhZuchtPlace:writeUpdateStream(streamId, connection, dirtyMask)
	if not connection:getIsServer() then
		streamWriteBool(streamId, self.SendAddCalb);
		self.SendAddCalb = false;
	end;
	KuhZuchtPlace:superClass().writeUpdateStream(self, streamId, connection, dirtyMask);
end;

function KuhZuchtPlace:update(dt)
	
	self.updateMs = (self.updateMs or 0) + (dt * g_currentMission.missionStats.timeScale);
	if self.updateMs >= 60000 then
		self.updateMs = self.updateMs - 60000;
		local MilkFillLevel = self:getFillLevel(Fillable.FILLTYPE_MILK);
		for i = 1, 5 do
			if self.Kaelber[i].has then
				if MilkFillLevel > 0 then
					MilkFillLevel = MilkFillLevel - (80/24/60);
					self.Kaelber[i].t = self.Kaelber[i].t - 1;
					if self.Kaelber[i].t <= 0 then
						self.Kaelber[i].has = nil;
						setVisibility(self.Kaelber[i].vis,false);
						g_currentMission.husbandries.cow:addAnimals(1,0);
					end;
				end;
			end;
		end;
		if MilkFillLevel ~= self:getFillLevel(Fillable.FILLTYPE_MILK) then
			self:setFillLevel(MilkFillLevel, Fillable.FILLTYPE_MILK);
		end;
	end;

end;

function KuhZuchtPlace:getSaveAttributesAndNodes(nodeIdent)

	local x,y,z = unpack(self.pos);
	local rx,ry,rz = unpack(self.rot);
	local Milk = KuhZuchtZE.MilkFillLevel or 0;
	local minute = KuhZuchtZE.updateMin or 0;
	
	local attributes = 'filename="'..Utils.encodeToHTML(Utils.convertToNetworkFilename(self.xml))..'" position="'..x..' '..y..' '..z..'" rotation="'..rx..' '..ry..' '..rz..'" Milk="'..Milk..'" minute="'..minute..'"';
    
	local nodes = "";
	local KalbNum = 0;
	
	for i = 1,table.getn(self.Kaelber) do
		if self.Kaelber[i].has then
			if KalbNum>0 then
				nodes = nodes.."\n";
			end;
			nodes = nodes..nodeIdent..'<kalb time="'..self.Kaelber[i].t..'" />';
			KalbNum = KalbNum+1;
		end;
	end;
	
    return attributes,nodes;
end;

function KuhZuchtPlace:loadFromAttributesAndNodes(xmlFile, Key)
	
	local xmlFilename =  Utils.convertFromNetworkFilename(getXMLString(xmlFile, Key.."#filename"));
	local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, Key.."#position"));
	local rx,ry,rz = Utils.getVectorFromString(getXMLString(xmlFile, Key.."#rotation"));
	
	if self:load(xmlFilename, x,y,z, rx,ry,rz) then
			
		KuhZuchtZE.MilkFillLevel = Utils.getNoNil(getXMLFloat(xmlFile, Key.."#Milk"),0);
		KuhZuchtZE.updateMin = Utils.getNoNil(getXMLFloat(xmlFile, Key.."#minute"),0);
		
		local i=0;
		while true do
			local kalbKey = Key..string.format(".kalb(%d)", i);
			if not hasXMLProperty(xmlFile, kalbKey) then
				break;
			end;
			local t = getXMLFloat(xmlFile, kalbKey.."#time");
			if t ~= nil then
				self.Kaelber[i+1].has = true;
				self.Kaelber[i+1].t = t;
				setVisibility(self.Kaelber[i+1].vis,true);
			end;
			i=i+1;
		end;
	end;
	
	return true;
end;

function KuhZuchtPlace:setFillLevel(fillLevel, fillType, UpdateCapa)
	
	if KuhZuchtZE and fillType == Fillable.FILLTYPE_MILK then
		if not UpdateCapa and fillLevel > KuhZuchtZE:getFillLevel() then
			self:sendFillLevel(fillLevel);
		else
			KuhZuchtZE:setFillLevel(fillLevel, UpdateCapa);
		end;
	end;
	
end;

function KuhZuchtPlace:getFillLevel(filltype)
	if KuhZuchtZE and filltype == Fillable.FILLTYPE_MILK then
		return  KuhZuchtZE:getFillLevel();
	end
	return 0;
end;

function KuhZuchtPlace:addKalb()
	for i = 1, 5 do
		if not self.Kaelber[i].has then
			self.Kaelber[i].has = true;
			self.Kaelber[i].t = 4320; -- 3 days in min
			setVisibility(self.Kaelber[i].vis,true);
			if self.isServer then
				self.SendAddCalb = true;
				self:raiseDirtyFlags(self.AddDirtyFlag);
			elseif KuhZuchtZE then
				KuhZuchtZE.updateMin = 0;
			end;
			break;
		end;
	end;
	
end;

function KuhZuchtPlace:sendFillLevel(fillLevel, noEventSend)
	 KuhZuchtUpdateFillLvlEvent.sendEvent(self, fillLevel, noEventSend);
	 KuhZuchtZE:setFillLevel(fillLevel);
end

function KuhZuchtPlace:TriggerCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
		
	local vehicle = g_currentMission.nodeToVehicle[otherId];
	if vehicle ~= nil and vehicle.addMilchTrailerFillTrigger ~= nil and vehicle.addMilchTrailerFillTrigger ~= nil then
		if onEnter then
			vehicle:addMilchTrailerFillTrigger({id = triggerId, Zucht = self});
		else -- onLeave
			vehicle:removeMilchTrailerFillTrigger({id = triggerId, Zucht = self});
		end;
	end;
	
 end;
 
registerPlaceableType("KuhZuchtPlace", KuhZuchtPlace);